@@ -1,23 +1,28 @@
 import requests
 import json
 import os
+import sys
 from jsonschema import validate
 from jsonschema.exceptions import ValidationError
-
-from censius.schemas import register_model_schema, revise_model_schema, update_actual_schema, individual_log_schema, batch_log_schema , process_model_schema,register_dataset_schema,register_project_schema
+import pandas as pd
+from censius.schemas import get_model_health_schema,register_new_model_version_schema,register_model_schema, revise_model_schema, update_actual_schema, individual_log_schema, batch_log_schema , process_model_schema,register_dataset_schema,register_project_schema,log_explanations_schema
+from censius.utils import check_time_format
 
 
 
-BASE_URL="http://censius-logs-prod.us-east-1.elasticbeanstalk.com/v1"
+BASE_URL="http://censius-logs-prod1.us-east-1.elasticbeanstalk.com/v1"
 AMS_URL="http://ams-prod.us-east-1.elasticbeanstalk.com"
+EXPLAINATION_BASE_URL="http://explainability-prod.us-east-1.elasticbeanstalk.com/explanations"
+MONITORS_PROGRAMMATIC_BASE_URL="http://monitors-prod.us-east-1.elasticbeanstalk.com/v1/programmatic"
 
 # Models
 REGISTER_MODEL_URL = lambda : f"{AMS_URL}/models/register"
 REVISE_MODEL_URL = lambda : f"{AMS_URL}/models/revise"
 PROCESS_MODEL_URL = lambda : f"{AMS_URL}/models/schema-updation"
+REGISTER_NEW_MODEL_VERSION=lambda : f"{AMS_URL}/models/register_new_version"
 
 # Logs
-LOG_URL = lambda : f"{BASE_URL}/logs/"
+LOG_URL = lambda : f"{BASE_URL}/logs"
 UPDATE_ACTUAL_URL = lambda prediction_id : f"{BASE_URL}/logs/{prediction_id}/updateActual"
 
 # Dataset
@@ -26,6 +31,12 @@ REGISTER_DATASET_URL = lambda : f"{AMS_URL}/datasets/register"
 # Project
 REGISTER_PROJECT_URL = lambda: f"{AMS_URL}/projects/register-project"
 
+#Explainations
+EXPLAINATION_URL= lambda: f"{EXPLAINATION_BASE_URL}/insert_local"
+
+#Monitors
+GET_MODEL_HEALTH_URL=lambda: f"{MONITORS_PROGRAMMATIC_BASE_URL}/get_model_health"
+
 class CensiusClient(object):
     def __init__(self, api_key, tenant_id):
         if api_key == None or len(api_key) == 0:
@@ -33,9 +44,33 @@ class CensiusClient(object):
         if tenant_id == None or len(tenant_id) == 0:
             raise ValueError("You need to pass a tenant ID")
         self.api_key = api_key
-        
+        self.tenantKey=tenant_id       
 
     
+    def get_model_health(self, *args,**kwargs):
+        try:
+            validate(instance=kwargs, schema=get_model_health_schema)
+        except ValidationError as e:
+            return e.message
+
+        payload = json.dumps({k: v for k, v in {
+            "model_id": kwargs["model_id"],
+            "model_version": kwargs['model_version'],
+            "metrics": kwargs["metrics"],
+            "start_timestamp":kwargs["start_timestamp"],
+            "end_timestamp":kwargs["end_timestamp"],
+        }.items() if v})
+        headers = {
+            'Authorization': f'Bearer {self.api_key}',
+            'Content-Type': 'application/json',
+            'api_key':str(self.api_key),
+            "tenant_key":str(self.tenantKey)
+        }
+
+        response = requests.request("POST", GET_MODEL_HEALTH_URL(), headers=headers, data=payload)
+        return response.json()
+
+
     def register_project(self, *args,**kwargs):
         try:
             validate(instance=kwargs, schema=register_project_schema)
@@ -70,6 +105,54 @@ class CensiusClient(object):
         return self.__return_message(response)
 
 
+    def register_new_model_version(self, *args, **kwargs):
+        try:
+            validate(instance=kwargs, schema=register_new_model_version_schema)
+        except ValidationError as e:
+            return e.message
+
+
+
+        WindowSize=None
+        if "window_size" in kwargs:
+            WindowSize=kwargs["window_size"]
+
+        WindowStartTime=None
+        if "start_time" in kwargs:
+            WindowStartTime=kwargs["start_time"]
+
+        payload = json.dumps({k: v for k, v in {
+            "userDefinedModelID": kwargs['model_id'],
+            "version": kwargs['model_version'],
+            "datasetId": kwargs['training_info']['id'],
+            "targets": kwargs["targets"],
+            "features": kwargs["features"],
+            "apiKey":self.api_key,
+            "windowSize":WindowSize,
+            "window_start_time":WindowStartTime,
+            "tenantKey":self.tenantKey
+        }.items() if v})
+        headers = {
+            'Authorization': f'Bearer {self.api_key}',
+            'Content-Type': 'application/json'
+        }
+
+        response = requests.request("POST", REGISTER_NEW_MODEL_VERSION(), headers=headers, data=payload)
+        if 'error' in response.json().keys():
+            return self.__return_message(response)
+        else:
+            values=[]
+            for i in kwargs["targets"]:
+                temp={}
+                temp["target"] = i
+                values.append(temp)
+            primary_response = response
+            self.process_model(
+                dataset_id=kwargs['training_info']['id'],
+                model_id=response.json()["message"]["ID"],
+                values=values
+            )
+            return self.__return_message(primary_response)
 
 
     def register_dataset(self, *args, **kwargs):
@@ -82,21 +165,48 @@ class CensiusClient(object):
         if 'raw_values' in kwargs:
             rawValues=str(kwargs["raw_values"])
 
+        timestampBase=None
         timestampCol=None
-        if 'timestamp_col' in kwargs:
-            timestampCol=kwargs["timestamp_col"]
-        
-        timestampType=None
-        if 'timestamp_type' in kwargs:
-            timestampType=kwargs["timestamp_type"]
-
         unixInterval=None
-        if 'unix_interval' in kwargs:
-            unixInterval=kwargs["unix_interval"]
+        timestampType=None
+        if 'timestamp' in kwargs:
+            timestampBase=kwargs["timestamp"]
+            timestampCol=timestampBase["name"]
+            if "iso" not in timestampBase["type"]:
+                timestampType="unix"
+                unixInterval=timestampBase["type"]
+            else:
+                timestampType=timestampBase["type"]
 
-        file=kwargs["file"]
+        Version=None
+        if 'version' in kwargs:
+            Version=kwargs["version"]
+
+        file=None
+        if 'file' in kwargs:
+            file=kwargs["file"]
+        file_path=None
+        if 'file_path' in kwargs:
+            file_path=kwargs["file_path"]
+        
         file_name=kwargs["name"]+".csv"
-        file.to_csv(file_name)
+        if file_path:
+            if '.csv' not in file_path:
+                return {"error":"Please provide a valid path for the csv file"}
+            try:
+                file=pd.read_csv(file_path)
+                if (file.memory_usage().sum())//1000000 > 500:
+                    return {"error":"Max allowed file size is 500 MB"}
+                file.to_csv(file_name)
+                
+            except Exception as e:
+                return {"error":"unable to read the csv file"}
+        elif file is not None:
+            if (file.memory_usage().sum())//1000000 > 500:
+                    return {"error":"Max allowed file size is 500 MB"}
+            file.to_csv(file_name)  
+        else:
+            return {"error":"Please provide a df object or a file path"}
 
         headers = {
             'Authorization': f'Bearer {self.api_key}'
@@ -104,9 +214,7 @@ class CensiusClient(object):
 
         payload={'name': kwargs["name"],
                 'projectId': kwargs['project_id'],
-                'type': kwargs["type"],
-                'version': kwargs["version"],
-                'createdBy': '1',
+                'version': Version,
                 'features': json.dumps(kwargs["features"]),
                 'apiKey':self.api_key,
                 'timestampCol':timestampCol,
@@ -134,8 +242,8 @@ class CensiusClient(object):
             return e.message
             
         WindowSize=None
-        if "windowsize" in kwargs:
-            WindowSize=kwargs["windowsize"]
+        if "window_size" in kwargs:
+            WindowSize=kwargs["window_size"]
 
         WindowStartTime=None
         if "window_start_time" in kwargs:
@@ -169,17 +277,32 @@ class CensiusClient(object):
         targets = kwargs['targets']
 
         features = kwargs['features']
-        
+
+        WindowSize=None
+        if "window_size" in kwargs:
+            WindowSize=kwargs["window_size"]
+
+        WindowStartTime=None
+        if "start_time" in kwargs:
+            WindowStartTime=kwargs["start_time"]
+
+        ModelName=None
+        if "model_name" in kwargs:
+            ModelName=kwargs["model_name"]
+
         payload = json.dumps({k: v for k, v in {
             "userDefinedModelID": kwargs['model_id'],
             "version": kwargs['model_version'],
-            "datasetId": kwargs['dataset_id'],
-            "name": kwargs['model_name'],
+            "datasetId": kwargs['training_info']['id'],
+            "name": ModelName,
             "projectId": kwargs['project_id'],
-            "type": kwargs['type'],
+            "type": kwargs['model_type'],
             "target": targets,
             "features": features,
             "apiKey":self.api_key,
+            "windowSize":WindowSize,
+            "window_start_time":WindowStartTime,
+            "tenantKey":self.tenantKey
         }.items() if v})
         headers = {
             'Authorization': f'Bearer {self.api_key}',
@@ -197,38 +320,68 @@ class CensiusClient(object):
                 values.append(temp)
             primary_response = response
             self.process_model(
-                dataset_id=kwargs["dataset_id"],
+                dataset_id=kwargs['training_info']['id'],
                 model_id=response.json()["message"]["ID"],
                 values=values
             )
             return self.__return_message(primary_response)
         
 
-    def revise_model(self, *args, **kwargs):
+    def update_model(self, *args, **kwargs):
         try:
             validate(instance=kwargs, schema=revise_model_schema)
         except ValidationError as e:
             return e.message
         
-        is_rolling_window_enabled = None
-        if 'is_rolling_window_enabled' in kwargs:
-            is_rolling_window_enabled = kwargs['is_rolling_window_enabled']
+        TrainingData=kwargs["training_info"]
+        IsRollingWindow=False
+        reMapped=False
+        MappedTo=None
+        WindowSize=None
+        WindowEndTime=None
+        WindowStartTime=None
+        
+        if "fixed" in TrainingData['method']:
+            IsRollingWindow=False
+            reMapped=False
+            if "start_time" in TrainingData.keys():
+                    WindowStartTime=TrainingData["start_time"]
+            if "end_time" in TrainingData.keys():
+                    WindowEndTime=TrainingData["end_time"]
+            
 
+        elif "id" in TrainingData["method"]:
+            IsRollingWindow=False
+            reMapped=True
+            MappedTo=TrainingData["id"]   
+        else:
+            IsRollingWindow=True
+            if "start_time" in TrainingData.keys():
+                    WindowStartTime=TrainingData["start_time"]
+            if "window_size" in TrainingData.keys():
+                WindowSize=TrainingData["window_size"]
+
+        
         payload = json.dumps({k: v for k, v in {
             "userDefinedModelID": kwargs['model_id'],
             "version": kwargs['model_version'],
-            "windowSize": kwargs['window_size'],
-            "isRollingWindowEnabled": is_rolling_window_enabled,
+            "isRollingWindow": IsRollingWindow,
             "apiKey":self.api_key,
-        }.items() if v})    
+            "windowSize":WindowSize,
+            "window_start_time":WindowStartTime,
+            "window_end_time":WindowEndTime,
+            "isRemapped":reMapped,
+            "remap_to":MappedTo
+        }.items() if v})
         headers = {
-            'Authorization': f'Bearer {self.api_key}',
+            'Authorization': "Bearer "+self.api_key,
             'Content-Type': 'application/json'
         }
-
+        
         response = requests.request("POST", REVISE_MODEL_URL(), headers=headers, data=payload)
         return self.__return_message(response)
 
+
     def log(self, *args, **kwargs):
         if len(args) > 0 and str(type(args[0])) == "<class 'list'>":
             return self.__batch_log(args[0])
@@ -249,6 +402,11 @@ class CensiusClient(object):
         if 'actual' in kwargs:
             actual = kwargs['actual']
 
+        rightFormat=check_time_format(kwargs['timestamp'])
+        if not rightFormat:
+            return "Timestamp is not of the Unix MS format"
+            
+
         payload = json.dumps({k: v for k, v in {
             "predictionID": kwargs['prediction_id'],
             "modelVersion": kwargs['model_version'],
@@ -322,6 +480,29 @@ class CensiusClient(object):
         response = requests.request("POST", UPDATE_ACTUAL_URL(kwargs['prediction_id']), headers=headers, data=payload)
         return self.__return_message(response)
 
+
+    def log_explanation(self, *args, **kwargs):
+        try:
+            validate(instance=kwargs, schema=log_explanations_schema)
+        except ValidationError as e:
+            return e.message
+
+        payload = json.dumps({k: v for k, v in {
+            "custom_model_id": kwargs['model_id'],
+            "model_version": kwargs['model_version'],
+            "log_id": kwargs["prediction_id"],
+            "apiKey":self.api_key,
+            "explanation_type":kwargs["explanation_type"],
+            "explanation_values":kwargs["explanation_values"]
+        }.items() if v})
+        headers = {
+            'Authorization': "Bearer "+self.api_key,
+            'Content-Type': 'application/json'
+        }
+        
+        response = requests.request("POST", EXPLAINATION_URL(), headers=headers, data=payload)
+        return self.__return_message(response)
+
     def __return_message(self, response):
         try:
             return response.json()['message']
