@@ -1,6 +1,95 @@
 from censius import ExplanationType, ModelType , DatasetType , Dataset
 
 
+get_model_health_schema ={
+    "type": "object",
+    "properties": {
+        "model_id": {
+            "type": "string"
+        },
+        "model_version": {
+            "type": "string"
+        },
+        "metrics": {
+            "type": "array",
+            "items": {
+                "type": "string",
+                "enum": ["performance", "explainability"]
+
+            }
+        },
+        "start_timestamp": {
+            "type": "number"
+        },
+        "end_timestamp": {
+            "type": "number"
+        }
+       
+        
+
+    },
+    "required": ["model_id", "model_version", "metrics","start_timestamp","end_timestamp"]
+
+
+}
+
+
+register_new_model_version_schema = {
+    "type": "object",
+    "properties": {
+        "model_id": {
+            "type": "string"
+        }
+        ,
+        "training_info":{
+            "type":"object",
+            "properties":{
+                "id":{
+                    "type":"integer"
+                },
+                "method":{
+                    "type":"string",
+                    "enum":[Dataset.ID]
+                }
+            },
+            "required": ["id", "method"]
+        },
+        "model_version": {
+            "type": "string"
+        },
+        "targets": {
+            "type": "array",
+            "items": {
+                "type": "string"
+            }
+        },
+        "features": {
+            "type": "array",
+            "items": {
+                "type": "string"
+            }
+        },
+        "window_size": {
+            "type": "object",
+            "properties": {
+                "number": {
+                    "type": "integer"
+                },
+                "unit": {
+                    "type": "string",
+                    "enum": ["day", "week", "hour"]
+                }
+            },
+            "required": ["number", "unit"]
+        },
+        "start_time": {
+            "type": "integer"
+        }
+    },
+    "required": ["training_info", "model_id", "model_version","targets","features"]
+}
+
+
 register_model_schema = {
     "type": "object",
     "properties": {
@@ -63,7 +152,7 @@ register_model_schema = {
             "type": "integer"
         }
     },
-    "required": ["training_info", "model_id", "model_name", "model_version", "project_id", "model_type","targets","features"]
+    "required": ["training_info", "model_id", "model_version", "model_name","project_id", "model_type","targets","features"]
 }
 register_dataset_schema = {
     "type": "object",
@@ -102,6 +191,9 @@ register_dataset_schema = {
         },
         "version": {
             "type": "string"
+        },
+        "file_path": {
+            "type": "string"
         },
          "timestamp": {
             "type": "object",
